-- Auther: aramsay
-- Create Date: 3/22/2009 9:10 PM
-- Status: Alpha
-- TODO:
--      (T) "stack" bar type
--      (T) "power" bar type
--      (T) "item" bar type
--      (T) moved create to it's own function
--      (T) frame offset stored in saved vars
--      (T) pass in barSettings to CreateBarFrame
--      (T) delete frame by name, not spell ID
--
--      (1) "energy" bar regen rate scaling for combat and when AR is up
--      (1) "stack" bar needs to track self debuffs (like arcane blast)
--      (1) default profiles
--      (1) bar management ui
--      (1) "totem" bar type
--      (1) "cast" bar type, don't specify a spell, just a regular cast bar w/ correct timing
--      (2) make GCD specific spells for each class
--      (2) localization
--      (2) "internal CD" bars
--      (2) "special" bar type(s), for Overpower/Taste For Blood, and other special abilities
--      (2) some way to handle DK runes
--      (2) setting to remove icons per frame
--      (2) allow a way to specify "stacking" bars, like WF/SS or Wolves/WolvesCD
--      (3) reuse bar frames
--      (3) multiple base frames
--      (3) debuff bar to check for buffs in same category (allow 2ndary,etc spells to be specified);
--      (3) allow multiple of the same spell to be stored
--      (3) detect spec change and load different profile/bars
--      (3) vertical marking (like GCD, or energy abilities)
--      (3) ticks
--      (3) export/import for sharing (can currently use macros as a work-around import)
--      (3) change bars for shape shift forms


AweInspired = LibStub("AceAddon-3.0"):NewAddon("AweInspired", "AceConsole-3.0", "AceEvent-3.0")

local media = LibStub:GetLibrary("LibSharedMedia-3.0")
local frameOffset = 0;
local stophide = false;
local bolMoveable = false;
local threshold = 1.5;
local numReset = 30;
local curBarName = "name";
local curSpellID = 1;
local curBarType = "cooldown";
local curBarColor = {r=1, g=1, b=1, a=1};
local curBarStackNum = 1;
local curLongBar = "wait";
				--"wait" holds the bar at full until it's remaining time is less than baseCD then starts
				--"scale" scales the bar to empty slower, losing the relative timing
				--"long" extends the bar beyond the width of baseCD and empties at the same rate of all bars.
				--"hybrid" runs scale while time left is > base CD, then refills and runs normal for the rest.

AweInspired.textures = {};
AweInspired.borders = {};

-----
-- Options passed to AceConfig-3.0 to enable slash commands.
-----
local options = {
	name = "AweInspired",
	handler = AweInspired,
	type = 'group',
	args = {
		deleteframe = {
			type = "input",
			name = "Delete Bar",
			desc = "Type in the spell ID of the bar to delete and press enter to remove it.",
			set = "DeleteFrame",
		},
		clearbars = {
			type = 'execute',
			name = "Clear Bars",
			desc = "Delete all bars",
			func = "ClearBars",
		},
		moveable = {
			type = 'toggle',
			name = "Move Bars",
			desc = "Enable bars to be dragged on the screen.",
			get = function() return bolMoveable; end,
			set = function(info, val)
					bolMoveable = val;
					AweInspired.BaseFrame:EnableMouse(bolMoveable);
					if bolMoveable then
						AweInspired.BaseFrame:SetBackdropColor(1, 1, 1, 1);
					else
						AweInspired.BaseFrame:SetBackdropColor(1, 1, 1, 0);
					end
				end,
			tristate = false,
		},
		general = {
			name = "General Options",
			type = 'group',
			args = {
				texture = {
					type = 'select',
					name = "Bar Texture",
					get = function() return AweInspired.db.profile.texture end,
					set = "SetBarTexture",
					values = AweInspired.textures,
				},
				border = {
					type = 'select',
					name = "Border Texture",
					get = function() return AweInspired.db.profile.border end,
					set = "SetBorderTexture",
					values = AweInspired.borders,
				},
				barheight = {
					type = "range",
					name = "Bar Height",
					desc = "The height of each bar.",
					min = 5,
					max = 100,
					step = 1,
					bigStep = 1,
					get = function(info) return AweInspired.db.profile.fHeight end,
					set = function(info,val) AweInspired.db.profile.fHeight = val end,
					isPercent = false,
				},
				barwidth = {
					type = "range",
					name = "Frame Width",
					desc = "Sets the width (in pixels) of bars with a cooldown equal to Base Cooldown (changing Long Bar settings may result in a wider frame).",
					min = 50,
					max = 1000,
					step = 1,
					bigStep = 1,
					get = function(info) return AweInspired.db.profile.fWidth end,
					set = function(info,val) AweInspired.db.profile.fWidth = val end,
					isPercent = false,
				},
				basecd = {
					type = "range",
					name = "Base Cooldown",
					desc = "The number in seconds of the max size bar, anything longer than this will be handled based on Long Bar settings.",
					min = 1,
					max = 60,
					step = 0.01,
					bigStep = 1,
					get = function(info) return AweInspired.db.profile.baseCD end,
					set = function(info,val) AweInspired.db.profile.baseCD = val end,
					isPercent = false,
				},
				scale = {
					type = "range",
					name = "Scale",
					desc = "UI Scale of the bars.",
					min = 0.1,
					max = 2,
					step = 0.01,
					bigStep = 0.05,
					get = function(info) return AweInspired.db.profile.scale; end,
					set = function(info,val) 
							AweInspired.BaseFrame:SetScale(val);
							AweInspired.db.profile.scale = val;
						end,
					isPercent = true,
				}
			},
		},
		newframe = {
			name = "Create Frame",
			type = 'group',
			args = {
				name = {
					type = "input",
					name = "Name",
					desc = "Name for the bar, only used internally or to delete the bar later",
					set = function(info,val) curBarName = val; end,
					get = function(info) return curBarName; end,
				},
				spellID = {
					type = "input",
					name = "Spell/Item ID",
					desc = "Spell ID or Item ID for new frame.",
					set = function(info,val) curSpellID = val; end,
					get = function(info) return curSpellID; end,
				},
				bartype = {
					type = "select",
					name = "Bar Type",
					desc = "Type of bar",
					values = {
						["cooldown"]="Cooldown",
						["buff"]="Buff",
						["debuff"]="Debuff",
						["gcd"]="GCD",
						["combo"]="Combo Points",
						["energy"]="Energy (experimental)",
						["power"]="Power (experimental)",
						["item"]="Item (experimental)",
						["stack"]="Stack (experimental)",
					},
					set = function(info,val) curBarType = val end,
					get = function(info) return curBarType end,
					style = "dropdown",
				},
				barcolor = {
					type = "color",
					name = "Bar Color",
					desc = "Color of the bar.",
					set = function(info,r,g,b,a)
							curBarColor = {r=r, g=g, b=b, a=a};
						end,
					get = function(info)
							return curBarColor.r, curBarColor.g, curBarColor.b, curBarColor.a;
						end,
					hasAlpha = true,
				},
				fullstack = {
					type = "input",
					name = "Full Stack Amount",
					desc = "For a \"stack\ type bar, this is how many stacks should be treated as a full bar, for any other bar type, this value is ignored.",
					set = function(info, val) curBarStackNum = val; end,
					get = function(info) return curBarStackNum; end,
				},
				longbar = {
					type = "select",
					name = "Long Bar",
					desc = "When the cooldown is longer than Base Cooldown, how to treat the bar \n\n\"Wait\" - Keep the bar full until the remaining duration is less than Base Cooldown.\n\n\"Scale\" - Scale the time down to fit in the same width, the relative speed of the bars is no longer constant.\n\n\"Long\" - Extend the width of the bar to represent the entire duration/cooldown of the spell.\n\n\"Hybrid\" - Run the spell as \"Scaled\" until the remaining time is less than Base Cooldown, then refill the bar and behave like \"Wait\" for the remainder of the duration.",
					values = {
						["wait"]="Wait",
						["scale"]="Scale",
						["long"]="Long",
						["hybrid"]="Hybrid",
					},
					get = function(info) return curLongBar end,
					set = function(info,val) curLongBar = val end,
					style = "dropdown",
				},
				create = {
					type = "execute",
					name = "Create",
					desc = "Create the bar with the specified settings.",
					func = "CreateBar",
				},
			},
		},
		managebars = {
			name = "Manage Bars",
			type = 'group',
			args = {
			},
		},
	},
};

-----
-- Default values for saved variables data, used with AceDB-3.0
-----
local defaults = {
	profile = {
		texture = "Blizzard",
		border = "None",
		fHeight = 25,
		fWidth = 300,
		baseCD = 10, --The width that the bar frames should use
		point = "CENTER",
		relativeTo = "UIParent",
		relativePoint = "CENTER",
		xOffset = 0,
		yOffset = 0,
		scale = 1,
		bars = {
			['**'] = {
				name = "name",
				spellID = 1,
				barType = "cooldown",
				color = {r=1, g=1, b=1, a=1},
				icon = true,
				longBar = "wait", --what to do if the bar needs to be longer than baseCD,
						--"wait" holds the bar at full until it's remaining time is less than baseCD then starts
						--"scale" scales the bar to empty slower, losing the relative timing
						--"long" extends the bar beyond the width of baseCD and empties at the same rate of all bars.
						--"hybrid" runs scale while time left is > base CD, then refills and runs normal for the rest.
				frameOffset = -1,
				stackNum = 1,
			},
		},
	},
};

local classDefaults = {
	["arcMage"] = {
		cooldowns = {
			[1] = {
				spellID = 12042,
				barType = "cooldown",
				color = {r=0, g=0, b=0.5, a=1},
				icon = true,
				name = "arcanepower",
				longBar = "hybrid",
			},
			[2] = {
				spellID = 12472,
				barType = "cooldown",
				color = {r=0.15, g=0.45, b=0.95, a=1},
				icon = true,
				name = "icyveins",
				longBar = "hybrid",
			},
			[3] = {
				spellID = 12043,
				barType = "cooldown",
				color = {r=0.5, g=0.5, b=0.5, a=1},
				icon = true,
				name = "presenceofmind",
				longBar = "hybrid",
			},
			[4] = {
				spellID = 12051,
				barType = "cooldown",
				color = {r=0.71, g=0.63, b=0.95, a=1},
				icon = true,
				name = "evocation",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 54490,
				barType = "buff",
				color = {r=0.8, g=0.15, b=0.8, a=1},
				icon = true,
				name = "missilebarrage",
				longBar = "wait",
			},
			[2] = {
				spellID = 36032,
				barType = "stack",
				color = {r=0.2, g=0, b=0.75, a=1},
				icon = true,
				name = "arcaneblaststack",
				longBar = "wait",
			},
		},
	},
	["fireMage"] = {
		cooldowns = {
			[1] = {
				spellID = 11129,
				barType = "cooldown",
				color = {r=0.9, g=0.67, b=0.15, a=1},
				icon = true,
				name = "combustion",
				longBar = "hybrid",
			},
			[2] = {
				spellID = 12051,
				barType = "cooldown",
				color = {r=0.71, g=0.63, b=0.95, a=1},
				icon = true,
				name = "evocation",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 22959,
				barType = "debufftype",
				color = {r=0.75, g=0.1, b=0.05, a=1},
				icon = true,
				name = "impscorch",
				longBar = "wait",
			},
			[1] = {
				spellID = 55360,
				barType = "debuff",
				color = {r=0.8, g=0.4, b=0, a=1},
				icon = true,
				name = "livingbomb",
				longBar = "wait",
			},
			[1] = {
				spellID = 48108,
				barType = "buff",
				color = {r=0.65, g=0.32, b=0.18, a=1},
				icon = true,
				name = "hotstreak",
				longBar = "wait",
			},
		},
	},
	["frostMage"] = {
		cooldowns = {
			[1] = {
				spellID = 12472,
				barType = "cooldown",
				color = {r=0.15, g=0.45, b=0.95, a=1},
				icon = true,
				name = "icyveins",
				longBar = "hybrid",
			},
			[2] = {
				spellID = 11958,
				barType = "cooldown",
				color = {r=0, g=0, b=0.5, a=1},
				icon = true,
				name = "coldsnap",
				longBar = "hybrid",
			},
			[3] = {
				spellID = 12051,
				barType = "cooldown",
				color = {r=0.71, g=0.63, b=0.95, a=1},
				icon = true,
				name = "evocation",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 44572,
				barType = "cooldown",
				color = {r=0.41, g=0.69, b=0.79, a=1},
				icon = true,
				name = "deepfreeze",
				longBar = "wait",
			},
			[2] = {
				spellID = 44544,
				barType = "buff",
				color = {r=0.18, g=0.36, b=0.7, a=1},
				icon = true,
				name = "fingersoffrost",
				longBar = "wait",
			},
			[3] = {
				spellID = 57761,
				barType = "buff",
				color = {r=0.49, g=0.43, b=0.68, a=1},
				icon = true,
				name = "brainfreeze",
				longBar = "wait",
			},
		}
	},
	["affLock"] = {
		cooldowns = {
		},
		spells = {
			[1] = {
				spellID = 63321,
				barType = "buff",
				color = {r=0.44, g=0.8, b=0.94, a=1},
				icon = true,
				name = "lifetap",
				longBar = "wait",
			},
			[2] = {
				spellID = 59164,
				barType = "cooldown",
				color = {r=0.52, g=0.83, b=0.36, a=1},
				icon = true,
				name = "haunt",
				longBar = "wait",
			},
			[3] = {
				spellID = 47843,
				barType = "debuff",
				color = {r=0.45, g=0.07, b=0.53, a=1},
				icon = true,
				name = "unstableaffliction",
				longBar = "wait",
			},
			[4] = {
				spellID = 47864,
				barType = "debuff",
				color = {r=0.74, g=0.2, b=0.02, a=1},
				icon = true,
				name = "curseofagony",
				longBar = "wait",
			},
			[5] = {
				spellID = 32391,
				barType = "debuff",
				color = {r=0.53, g=0.6, b=1, a=1},
				icon = true,
				name = "shadowsembrace",
				longBar = "wait",
			},
		},
	},
	["demoLock"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["destroLock"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["discPriest"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["holyPriest"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["shadowPriest"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["mutRogue"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["combatRogue"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["subRogue"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["moonkinDruid"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["feralBearDruid"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["feralCatDruid"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["restoDruid"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["beastHunter"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["marksHunter"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["survHunter"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["elemShaman"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["enhShaman"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["restoShaman"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["armsWarrior"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["furyWarrior"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["protWarrior"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["holyPally"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["protPally"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["retPally"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["bloodDPSdk"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["frostDPSdk"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["unholyDPSdk"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["bloodTankDk"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["frostTankDk"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
	["unholyTankDk"] = {
		cooldowns = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "hybrid",
			},
		},
		spells = {
			[1] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=0.00, g=0.00, b=0.00, a=1},
				icon = true,
				name = "",
				longBar = "wait",
			},
		},
	},
};

AweInspired.frames = {};
AweInspired.barBackdrop = {
	--bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = false, tileSize = 0, edgeSize = 12,
	insets = { left = 2, right = 2, top = 2, bottom = 2 }
};

AweInspired.frameBackdrop = {
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = false, tileSize = 0, edgeSize = 12,
	insets = { left = 0, right = 0, top = 0, bottom = 0 }
};

AweInspired.BaseFrame = CreateFrame("Frame", "AweInspiredBase", UIParent)


-----
-- Initialization, fires when logging on or reloading UI.
-- Register the slash commands, and the bliz interface options menu using AceConfig
-- Set up the saved variables self.db wiht AceDB-3.0.
-- Create the base frame that will be the anchor for the bar frames.
-- Register shared media, i have no idea how to do this yet
-----
function AweInspired:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("AweInspiredDB", defaults);
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AweInspired", options);
	media.RegisterCallback(self, "LibSharedMedia_Registered");
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AweInspired", "AweInspired");
	self:RegisterChatCommand("aweinspired", "ChatCommand");
	self:RegisterChatCommand("ais", "ChatCommand");
	self:Print("AweInspired loaded");
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig");
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig");
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig");
end

-----
-- I have no idea what this does, I copied it from ShockAndAwe to try and incorporate SharedMedia
-----
function AweInspired:LibSharedMedia_Registered()
	for k, v in pairs(media:List("statusbar")) do
		self.textures[v] = v;
	end
	for k, v in pairs(media:List("border")) do
		self.borders[v] = v
	end
end

-----
-- Fired when you enable the mod, register all event handlers here
-----
function AweInspired:OnEnable()
	self:LibSharedMedia_Registered();
	--self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");

	self:RefreshConfig();
end

-----
-- Fired when you disable the mod, unregister all event handlers here, so they don't fire while disabled
-----
function AweInspired:OnDisable()
	--self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:UnregisterEvent("PLAYER_REGEN_DISABLED");
	self:UnregisterEvent("PLAYER_REGEN_ENABLED");
end


-----
-- This function fires when exiting combat.  Disable all bars.
-----
function AweInspired:PLAYER_REGEN_ENABLED()
	for i,v in pairs(self.frames) do
		v:Hide();
		v.icon:Hide();
	end
end

-----
-- This function fires when entering combat, enable all bars.
-----
function AweInspired:PLAYER_REGEN_DISABLED()
	for i,v in pairs(self.frames) do
		--self:Print("showing index ["..i.."]");
		v:Show();
		v.icon:Show();
	end
end

-----
-- Create the base frame, this will be the anchor for the other bar frames.
-- I'm not 100% certain exactly what all this does, but it should allow me to relocate where the bars end up.
-- The logic was copied from ShockAndAwe.
-----
function AweInspired:CreateBaseFrame()
	self.BaseFrame:SetScale(self.db.profile.scale);
	self.BaseFrame:SetFrameStrata("BACKGROUND");
	self.BaseFrame:SetWidth(300 + 175 / 7);
	self.BaseFrame:SetHeight(175);
	self.BaseFrame:SetBackdrop(self.frameBackdrop);
	self.BaseFrame:SetBackdropColor(1, 1, 1, 0);
	self.BaseFrame:SetMovable(true);
	self.BaseFrame:RegisterForDrag("LeftButton");
	self.BaseFrame:SetPoint(self.db.profile.point, self.db.profile.relativeTo, self.db.profile.relativePoint, self.db.profile.xOffset, self.db.profile.yOffset);
	self.BaseFrame:SetScript("OnDragStart",
		function()
			self.BaseFrame:StartMoving();
		end );
	self.BaseFrame:SetScript("OnDragStop",
		function()
			self.BaseFrame:StopMovingOrSizing();
			self.BaseFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.profile, self.BaseFrame);
		end ) ;
	self.BaseFrame:Show()
end


function AweInspired:FinishedMoving(var,frame)
	local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint();
	var.point = point
	var.relativeTo = relativeTo
	var.relativePoint = relativePoint
	var.xOffset = xOffset
	var.yOffset = yOffset
end


-----
-- Creates a new bar frame.  This will be located relative to the base frame and offset determined by
-- the value of the frameOffset variable.  This creates the statusbar and spark for the bar as well as
-- setting the overall size.
--
-- Parameters:
--   spellID: the spell ID of the frame to add.
--   frameColor: The RGB and transparency values for the color of the statusbar.
--   icon: boolean to determine if an Icon should be displayed for this bar.
-----
--function AweInspired:CreateBarFrame(name, spellID, barType, frameColor, icon, frameLongBar)
function AweInspired:CreateBarFrame(barSettings, saveBar)
	local name = barSettings.name;
	local spellName, spellIcon;
	local idType = self:GetIDType(barSettings.barType)

	if saveBar then
		table.insert(self.db.profile.bars, barSettings);
	end

	if idType == "spellID" then
		spellName, _, spellIcon, _, _, _, _, _, _ = GetSpellInfo(barSettings.spellID);
	elseif idType == "itemID" then
		spellName, _, _, _, _, _, _, _, _, spellIcon, _ = GetItemInfo(barSettings.spellID)
	else
		spellName = "";
		spellIcon = "";
		barSettings.icon = false;
	end

	--Create the frame and icon frame
	self.frames[name] = CreateFrame("Frame", name, self.BaseFrame);
	self.frames[name].icon = CreateFrame("Frame", name.."Icon", self.BaseFrame);

	--Store some frame specific variables
	self.frames[name].spellID = barSettings.spellID;
	self.frames[name].spellName = spellName;
	self.frames[name].barType = barSettings.barType;
	self.frames[name].longBar = barSettings.frameLongBar;
	self.frames[name].lastVal = 0;
	self.frames[name].numReset = 0;
	if barSettings.barType == "gcd" then
		self.frames[name].Width = (1.5/self.db.profile.baseCD) * self.db.profile.fWidth;
		self.frames[name].MaxVal = 1.5;
	else
		self.frames[name].Width = self.db.profile.fWidth;
		self.frames[name].MaxVal = self.db.profile.baseCD;
	end

	if barSettings.barType == "stack" then
		self.frames[name].stackNum = barSettings.stackNum
	else
		self.frames[name].stackNum = 0;
	end

	--Set the values of the base frame
	self.frames[name]:SetFrameStrata("LOW");
	self.frames[name]:SetWidth(self.frames[name].Width);
	self.frames[name]:SetHeight(self.db.profile.fHeight);
	self.frames[name]:ClearAllPoints();
	self.frames[name]:SetPoint("TOPLEFT", self.BaseFrame, "TOPLEFT", self.db.profile.fHeight, barSettings.frameOffset * ((-1 * self.db.profile.fHeight) + 3));
	self.frames[name]:SetBackdrop(self.barBackdrop);
	self.frames[name]:SetBackdropColor(0,0,0,0.2);
	self.frames[name]:SetBackdropBorderColor(1,1,1,1);
	self.frames[name]:EnableMouse(false);

	--Create the status bar and set the values
	if not self.frames[name].statusbar then
		self.frames[name].statusbar = CreateFrame("StatusBar", nil, self.frames[name])
	end
	self.frames[name].statusbar:SetFrameLevel(1);
	self.frames[name].statusbar:ClearAllPoints();
	self.frames[name].statusbar:SetHeight(self.db.profile.fHeight - 6);
	self.frames[name].statusbar:SetWidth(self.frames[name].Width - 6);
	self.frames[name].statusbar:SetPoint("TOPLEFT", self.frames[name], "TOPLEFT", 3, -3);
	--self.frames[name].statusbar:SetStatusBarTexture("Interface/Tooltips/UI-Tooltip-Background");
	self.frames[name].statusbar:SetStatusBarTexture(media:Fetch("statusbar", self.db.profile.texture));
	self.frames[name].statusbar:SetStatusBarColor(barSettings.color.r, barSettings.color.g, barSettings.color.b, barSettings.color.a);
	self.frames[name].statusbar:SetMinMaxValues(0, self.frames[name].MaxVal);
	self.frames[name].statusbar:SetValue(self.db.profile.baseCD);

	--Create the spark and set the values
	if not self.frames[name].spark then
		self.frames[name].spark = self.frames[name].statusbar:CreateTexture(nil, "OVERLAY");
	end
	self.frames[name].spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark");
	self.frames[name].spark:SetWidth(16);
	self.frames[name].spark:SetHeight(self.db.profile.fHeight + 16);
	self.frames[name].spark:SetBlendMode("ADD");
	self.frames[name].spark:SetPoint("CENTER",self.frames[name].statusbar,"LEFT",(self.frames[name].Width - 6), 0);
	self.frames[name].spark:Show();

	--Set the OnUpdate event of this frame to call OnBarUpdate
	self.frames[name]:SetScript("OnUpdate",
		function()
			AweInspired:OnBarUpdate(self.frames[name]);
		end );

	--If we're supposed to display an icon, set the values for that
	if barSettings.icon then
		self.frames[name].icon:SetWidth(self.db.profile.fHeight - 3)
		self.frames[name].icon:SetHeight(self.db.profile.fHeight - 3)
		self.frames[name].icon:SetPoint("TOPRIGHT", self.frames[name], "TOPLEFT", 0, -2)
		self.frames[name].icon:SetBackdrop({
			bgFile = spellIcon,
			--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = false, tileSize = 0, edgeSize = 12,
			insets = { left = 0, right = 0, top = 0, bottom = 0 } })
		self.frames[name].icon:Show();
	end

	self.frames[name]:Show();
	self.frames[name].spark:Show();
end


-----
-- This function returns the type of ID that is used for each barType, most use Spell ID but some use others.
-----
function AweInspired:GetIDType(barType)
	if barType == "cooldown"
			or barType == "debuff"
			or barType == "buff"
			or barType == "energy"
			or barType == "debufftype"
			or barType == "totem"
			or barType == "icd"
			or barType == "gcd"
			or barType == "stack" then
		return "spellID";
	elseif  barType == "item" then
		return "itemID";
	elseif barType == "rune" then
		return "runeID";
	else
		return "none";
	end
end


-----
-- Called constantly from the OnUpdate event of each bar, this is where the bars actually decrease in value over time.
-- the barFrame parameter should contain the frame that triggered the event, decrement it's value.
-----
function AweInspired:OnBarUpdate(barFrame)
	local totCD, timeLeft, count, expirationTime;

	if barFrame.barType == "cooldown" or barFrame.barType == "gcd" then
		timeLeft, totCD = self:GetCooldownValues(barFrame.spellID, barFrame.spellName, true, barFrame.MaxVal, false);
	elseif barFrame.barType == "buff" then
		timeLeft, totCD = self:GetBuffDebuffValues(true, barFrame.spellName, 0);
	elseif barFrame.barType == "debuff" then
		timeLeft, totCD = self:GetBuffDebuffValues(false, barFrame.spellName, 0);
	elseif barFrame.barType == "energy" then
		timeLeft, totCD = self:GetEnergyValues(barFrame.spellID);
	elseif barFrame.barType == "combo" then
		timeLeft, totCD = self:GetComboValues();
	elseif barFrame.barType == "power" then
		timeLeft, totCD = self:GetPowerValues();
	elseif barFrame.barType == "item" then
		timeLeft, totCD = self:GetCooldownValues(barFrame.spellID, barFrame.spellName, false, barFrame.MaxVal, true);
	elseif barFrame.barType == "stack" then
		timeLeft, totCD = self:GetBuffDebuffValues(true, barFrame.spellName, barFrame.stackNum);
		--If we didn't find the stack as a buff, check it as a debuff
		if totCD == 0 then
			timeLeft, totCD = self:GetBuffDebuffValues(false, barFrame.spellName, barFrame.stackNum);
		end
	else
		return;
	end

	--If the cooldown is shorter than the bar, shrink it down a bit.
	if totCD > 2 and totCD < barFrame.MaxVal then
		barFrame.Width = (totCD / barFrame.MaxVal) * barFrame.Width;
		barFrame:SetWidth(barFrame.Width);
		barFrame.statusbar:SetWidth(barFrame.Width - 6);
		barFrame.statusbar:SetMinMaxValues(0, totCD);
		barFrame.MaxVal = totCD;
	end

	--If there is more time left on the CD than the max value of the status bar, check the longBar setting to see what to do
	if totCD > barFrame.MaxVal then
		if barFrame.longBar == "wait" then
			if timeLeft > barFrame.MaxVal then
				timeLeft = barFrame.MaxVal;
			end
		elseif barFrame.longBar == "scale" then
			timeLeft = (barFrame.MaxVal * timeLeft) / totCD;
		elseif barFrame.longBar == "long" then
			barFrame.Width = (totCD / barFrame.MaxVal) * barFrame.Width;
			barFrame:SetWidth(barFrame.Width);
			barFrame.statusbar:SetWidth(barFrame.Width - 6);
			barFrame.statusbar:SetMinMaxValues(0, totCD);
			barFrame.MaxVal = totCD;
		elseif barFrame.longBar == "hybrid" then
			if timeLeft > barFrame.MaxVal then
				timeLeft = (barFrame.MaxVal * timeLeft) / totCD;
			end
		else
			--This shouldn't ever happen, but just to prevent errors if it does, treat is like "wait"
			if timeLeft > barFrame.MaxVal then
				timeLeft = barFrame.MaxVal;
			end
		end
	end

	if timeLeft <= 0 and barFrame.lastVal - timeLeft > threshold and barFrame.numReset <= numReset then
		timeLeft = barFrame.lastVal
		barFrame.numReset = barFrame.numReset + 1;
	else
		barFrame.numReset = 0;
	end
	
	local sparkLoc = (timeLeft / barFrame.MaxVal) * (barFrame.Width - 6);

	if sparkLoc > 0 then
		barFrame.spark:Show();
	else
		barFrame.spark:Hide();
	end
	
	barFrame.lastVal = timeLeft;
	barFrame.statusbar:SetValue(timeLeft);
	barFrame.spark:SetPoint("CENTER", barFrame.statusbar, "LEFT", sparkLoc, -1);
end


function AweInspired:GetCooldownValues(spellID, spellName, extendExecute, maxVal, isItem)
	local cdStart, totCD;
	if isItem then
		cdStart, totCD, _ = GetItemCooldown(spellID);
	else
		cdStart, totCD, _ = GetSpellCooldown(spellID);
	end
	local timeLeft = (cdStart + totCD) - GetTime();

	if extendExecute then
		local usable, nomana = IsUsableSpell(spellName);
		if not usable and not nomana and timeLeft < 1.5 then
			timeLeft = maxVal;
		end
	end
	return timeLeft, totCD;
end

function AweInspired:GetBuffDebuffValues(buff, spellName, stackNum)
	local unitCode, unitFilter, timeLeft;
	if buff then
		unitCode = "player";
		unitFilter = "HELPFUL|PLAYER";
	else
		unitCode = "target";
		unitFilter = "HARMFUL|PLAYER";
	end
	local _, _, _, count, _, totCD, expirationTime, _, _, _, _ = UnitAura(unitCode, spellName, nil, unitFilter);

	if totCD == nil then
		timeLeft = 0;
		totCD = 0
	elseif stackNum <= 0 then
		timeLeft = expirationTime - GetTime();
	else
		timeLeft = count * self.db.profile.baseCD / stackNum
		totCD = self.db.profile.baseCD;
	end

	return timeLeft, totCD;
end

function AweInspired:GetEnergyValues(spellID)
	local spellName, _, spellIcon, spellCost, _, spellPowerType, _, _, _ = GetSpellInfo(spellID);
	local energy = UnitMana("player");
	local powerType, _ = UnitPowerType("player");

	if powerType ~= 3 or spellPowerType ~= 3 then
		return 0, 0;
	end
	local timeLeft = (spellCost - energy) / 10.0;

	return timeLeft, (spellCost / 10.0);
end

function AweInspired:GetComboValues()
	local comboPoints = GetComboPoints("player", "target");
	return comboPoints * self.db.profile.baseCD / 5, self.db.profile.baseCD;
end

function AweInspired:GetPowerValues()
	local maxPower = UnitPowerMax("player");
	local curPower = UnitPower("player");
	return curPower * self.db.profile.baseCD / maxPower, self.db.profile.baseCD;
end


-----
-- This function is called with the /ais deleteframe command, it removes the frame for the given spell ID
-----
function AweInspired:DeleteFrame(info, frameName)
	if frameName == nil then
		frameName = 0;
	end

	for i,v in pairs(self.db.profile.bars) do
		if tostring(i) == frameName then
			table.remove(self.db.profile.bars, i);
		end
	end
	self:RefreshConfig();
end


function AweInspired:ClearBars()
	wipe(self.db.profile.bars);
	self:RefreshConfig();
end



-----
-- This function handles all chat /commands, if no value is specified it opens the interface options for this mod, otherwise it runs the command.
-----
function AweInspired:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(AweInspired, "ais", "AweInspired", input);
	end
end

-----
-- This is called when selecting an new texture from the options menu, it updates the textures of each bar.
-----
function AweInspired:SetBarTexture(info, value)
	self.db.profile.texture = value;
	local barTexture = media:Fetch("statusbar", value);
	for i,v in pairs(self.frames) do
		v.statusbar:SetStatusBarTexture(barTexture);
	end
end

-----
-- This is called when selecting an new border from the options menu, it updates the border of each bar.
-----
function AweInspired:SetBorderTexture(info, value)
	self.db.profile.border = value;
	local newTexture = media:Fetch("border", value);
	if newTexture then
		self.barBackdrop.edgeFile = newTexture;
		for i,v in pairs(self.frames) do
			v:SetBackdrop(self.barBackdrop);
		end
		self:CreateBaseFrame();
	end
end

-----
-- This function is called whenever a profile changes, any changes to saved variables that need to take place immediately should be in here.
-----
function AweInspired:RefreshConfig()
	self:SetBorderTexture(_, self.db.profile.border);
	--self.barBackdrop.edgeFile = media:Fetch("border",self.db.profile.border);


	--This stops the scripts from running.
	for i,v in pairs(self.frames) do
		v:SetScript("OnUpdate", nil);
		v:Hide();
		v.icon:Hide();
		v.spark:Hide();
	end

	--Clear out all existing frames, reset frameOffset so the positioning starts back at the top.
	wipe(self.frames);
	frameOffset = 0;
	self:CreateBaseFrame();

	--Add any saved frames from the new profile.
	for i,v in pairs(self.db.profile.bars) do
		--self:CreateBarFrame(v.name, v.spellID, v.barType, v.color, v.icon, v.longBar)
		if v.frameOffset == -1 then
			v.frameOffset = frameOffset;
		end
		self:CreateBarFrame(v,false);
		frameOffset = frameOffset + 1;
	end
end

function AweInspired:CreateBar()
	self:CreateBarFrame({name=curBarName, spellID=curSpellID, barType=curBarType, color=curBarColor, icon=true, longBar=curLongBar, frameOffset=frameOffset, stackNum=curBarStackNum}, true);
	--table.insert(self.db.profile.bars, {name = curBarName, spellID = curSpellID, barType = curBarType, color = curBarColor, icon = true, longBar = curLongBar});
	curBarName = "";
	curSpellID = 1;
	curBarType = "cooldown";
	curBarColor = {r=1, g=1, b=1, a=1};
	curBarStackNum = 1;
	frameOffset = frameOffset + 1;
end
